Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Definition Qsum l := fold_right Qplus 0%Q l.

Theorem Aux: forall a x, a >=0 -> a*x^2 >=0.
simpl.
intros K X.
intro H0.
rewrite Qmult_comm.
setoid_replace 0 with (0*K).
2:ring.
apply Qmult_le_compat_r;auto.
case (Qlt_le_dec 0 X).
intro H.
assert (H':=Qlt_le_weak _ _ H).
setoid_replace 0 with (0 * X).
2:ring.
apply Qmult_le_compat_r;auto.
intro H.
assert (H':-X >= 0).
setoid_replace 0 with (-0).
2:ring.
apply Qopp_le_compat;auto.
setoid_replace 0 with (0 * -X).
2:ring.
setoid_replace (X * X) with (-X * -X).
2:ring.
apply Qmult_le_compat_r;auto.
Qed.

Theorem Aux2: forall x y, 0 <= x -> 0 <= y -> 0 <= x + y.
intros x y Hx Hy.
setoid_replace 0 with (0+0) at 1; [apply Qplus_le_compat|ring].
exact Hx.
exact Hy.
Qed.

Ltac simplify_sos := match goal with 
 |- 0<= ?a + ?b => apply Aux2
 | |- 0 <= 0 => reflexivity
 | |- _ => apply Aux
| |- _=> compute;congruence
end.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros;apply Qmult_lt_0_le_reg_r with q.
elim (Qle_lt_or_eq _ _ H1).
auto.
intro H';symmetry in H';elim (H2 H').
rewrite H.
rewrite Qmult_0_l.
auto.
Qed.


Section Poly.
Variables  x y z w :Q.
(* POLYNOMIAL *)
Definition poly :=  x^2*y^2 + x^2*z^2 + y^2*z^2 - 4*x*y*z*w + w^4 .

(*SOLUTION *)

(* NUM *)
(* 6980476*x^4*y^2 + 8254164*x^2*y^4 + 6980476*x^4*z^2 + 37151880*x^2*y^2*z^2 + 8254164*y^4*z^2 + 21917240*x^2*z^4 + 21917240*y^2*z^4 - 27921904*x^3*y*z*w - 33016656*x*y^3*z*w - 87668960*x*y*z^3*w + 8128118*x^2*y^2*w^2 + 8128118*x^2*z^2*w^2 + 8128118*y^2*z^2*w^2 - 32512472*x*y*z*w^3 + 6980476*x^2*w^4 + 8254164*y^2*w^4 + 21917240*z^2*w^4 + 8128118*w^6 *)
(* summands *)
Definition num_decomp :=
( ( 8128118 # 1 ) * ( ( -1 # 1 ) * x*y*z + ( 1 # 1 ) * w^3 )^2) ::
( ( 21917240 # 1 ) * ( ( -4143571 # 21917240 ) * x^2*z + ( -1544599 # 4383448 ) * y^2*z + ( -5025337 # 10958620 ) * x*y*w + ( 1 # 1 ) * z*w^2 )^2) ::
( ( 8254164 # 1 ) * ( ( -4639091 # 8254164 ) * x^2*y + ( -6947989 # 8254164 ) * y*z^2 + ( 277743 # 687847 ) * x*z*w + ( 1 # 1 ) * y*w^2 )^2) ::
( ( 37470086 # 1 ) * ( ( -11578173 # 37470086 ) * x*y^2 + ( -24481553 # 37470086 ) * x*z^2 + ( 1 # 1 ) * y*z*w + ( -705180 # 18735043 ) * x*w^2 )^2) ::
( ( 132633942243239 # 8254164 ) * ( ( 3951227388865 # 132633942243239 ) * x^2*y + ( 1 # 1 ) * y*z^2 + ( -136585169632104 # 132633942243239 ) * x*z*w )^2) ::
( ( 24252768323467 # 4383448 ) * ( ( 12882570760699 # 24252768323467 ) * x^2*z + ( 1 # 1 ) * y^2*z + ( -37135339084166 # 24252768323467 ) * x*y*w )^2) ::
( ( 129784960355668 # 18735043 ) * ( ( -26613204405085 # 64892480177834 ) * x*y^2 + ( -38279275772749 # 64892480177834 ) * x*z^2 + ( 1 # 1 ) * x*w^2 )^2) ::
( ( 578138798154710153908 # 132633942243239 ) * ( ( -1 # 1 ) * x^2*y + ( 1 # 1 ) * x*z*w )^2) ::
( ( 227863584592896124675 # 64892480177834 ) * ( ( -1 # 1 ) * x*y^2 + ( 1 # 1 ) * x*z^2 )^2) ::
( ( 562181836211126069636 # 121263841617335 ) * ( ( -1 # 1 ) * x^2*z + ( 1 # 1 ) * x*y*w )^2) ::
nil.

(* DEN *)
(* 6980476*x^2 + 8254164*y^2 + 21917240*z^2 + 8128118*w^2 *)
(* summands *)
Definition denum_decomp :=
( ( 8128118 # 1 ) * ( ( 1 # 1 ) * w )^2) ::
( ( 21917240 # 1 ) * ( ( 1 # 1 ) * z )^2) ::
( ( 8254164 # 1 ) * ( ( 1 # 1 ) * y )^2) ::
( ( 6980476 # 1 ) * ( ( 1 # 1 ) * x )^2) ::
nil.

Lemma Ident: poly * Qsum (denum_decomp) == Qsum (num_decomp).
intros;simpl.
unfold poly.
Time ring.                            
Qed.

Lemma num_pos: Qsum (num_decomp) >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma  denum_pos: Qsum denum_decomp >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma T:~Qsum denum_decomp == 0 -> poly >=0.
intros.
apply (non_critical _ _ _ Ident);
auto using num_pos,denum_pos.
Qed.

End Poly.


Check T.
