Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Definition Qsum l := fold_right Qplus 0%Q l.

Theorem Aux: forall a x, a >=0 -> a*x^2 >=0.
simpl.
intros K X.
intro H0.
rewrite Qmult_comm.
setoid_replace 0 with (0*K).
2:ring.
apply Qmult_le_compat_r;auto.
case (Qlt_le_dec 0 X).
intro H.
assert (H':=Qlt_le_weak _ _ H).
setoid_replace 0 with (0 * X).
2:ring.
apply Qmult_le_compat_r;auto.
intro H.
assert (H':-X >= 0).
setoid_replace 0 with (-0).
2:ring.
apply Qopp_le_compat;auto.
setoid_replace 0 with (0 * -X).
2:ring.
setoid_replace (X * X) with (-X * -X).
2:ring.
apply Qmult_le_compat_r;auto.
Qed.

Theorem Aux2: forall x y, 0 <= x -> 0 <= y -> 0 <= x + y.
intros x y Hx Hy.
setoid_replace 0 with (0+0) at 1; [apply Qplus_le_compat|ring].
exact Hx.
exact Hy.
Qed.

Ltac simplify_sos := match goal with 
 |- 0<= ?a + ?b => apply Aux2
 | |- 0 <= 0 => reflexivity
 | |- _ => apply Aux
| |- _=> compute;congruence
end.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros;apply Qmult_lt_0_le_reg_r with q.
elim (Qle_lt_or_eq _ _ H1).
auto.
intro H';symmetry in H';elim (H2 H').
rewrite H.
rewrite Qmult_0_l.
auto.
Qed.


Section Poly.
Variables  x y z :Q.
(* POLYNOMIAL *)
Definition poly :=  x^4*y^2 - 3*x^2*y^2*z^2 + y^4*z^2 + x^2*z^4 .

(*SOLUTION *)

(* NUM *)
(* 8128145*x^6*y^2 + 8128146*x^4*y^4 - 1934498*x^4*y^2*z^2 - 16256293*x^2*y^4*z^2 + 8128146*y^6*z^2 + 8128145*x^4*z^4 - 59221665*x^2*y^2*z^4 + 22449937*y^4*z^4 + 22449937*x^2*z^6 *)
(* summands *)
Definition num_decomp :=
( ( 22449937 # 1 ) * ( ( -11224969 # 22449937 ) * x^2*y^2 + ( -11224968 # 22449937 ) * x^2*z^2 + ( 1 # 1 ) * y^2*z^2 )^2) ::
( ( 8128146 # 1 ) * ( ( -1 # 1 ) * x^2*y*z + ( 1 # 1 ) * y^3*z )^2) ::
( ( 22449937 # 1 ) * ( ( -1 # 1 ) * x*y^2*z + ( 1 # 1 ) * x*z^3 )^2) ::
( ( 8128145 # 1 ) * ( ( -1 # 1 ) * x^3*y + ( 1 # 1 ) * x*y*z^2 )^2) ::
( ( 56476436575841 # 22449937 ) * ( ( -1 # 1 ) * x^2*y^2 + ( 1 # 1 ) * x^2*z^2 )^2) ::
nil.

(* DEN *)
(* 8128145*x^2 + 8128146*y^2 + 22449937*z^2 *)
(* summands *)
Definition denum_decomp :=
( ( 22449937 # 1 ) * ( ( 1 # 1 ) * z )^2) ::
( ( 8128146 # 1 ) * ( ( 1 # 1 ) * y )^2) ::
( ( 8128145 # 1 ) * ( ( 1 # 1 ) * x )^2) ::
nil.

Lemma Ident: poly * Qsum (denum_decomp) == Qsum (num_decomp).
intros;simpl.
unfold poly.
Time ring.                            
Qed.

Lemma num_pos: Qsum (num_decomp) >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma  denum_pos: Qsum denum_decomp >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma T:~Qsum denum_decomp == 0 -> poly >=0.
intros.
apply (non_critical _ _ _ Ident);
auto using num_pos,denum_pos.
Qed.

End Poly.


Check T.
