Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Definition Qsum l := fold_right Qplus 0%Q l.

Theorem Aux: forall a x, a >=0 -> a*x^2 >=0.
simpl.
intros K X.
intro H0.
rewrite Qmult_comm.
setoid_replace 0 with (0*K).
2:ring.
apply Qmult_le_compat_r;auto.
case (Qlt_le_dec 0 X).
intro H.
assert (H':=Qlt_le_weak _ _ H).
setoid_replace 0 with (0 * X).
2:ring.
apply Qmult_le_compat_r;auto.
intro H.
assert (H':-X >= 0).
setoid_replace 0 with (-0).
2:ring.
apply Qopp_le_compat;auto.
setoid_replace 0 with (0 * -X).
2:ring.
setoid_replace (X * X) with (-X * -X).
2:ring.
apply Qmult_le_compat_r;auto.
Qed.

Theorem Aux2: forall x y, 0 <= x -> 0 <= y -> 0 <= x + y.
intros x y Hx Hy.
setoid_replace 0 with (0+0) at 1; [apply Qplus_le_compat|ring].
exact Hx.
exact Hy.
Qed.

Ltac simplify_sos := match goal with 
 |- 0<= ?a + ?b => apply Aux2
 | |- 0 <= 0 => reflexivity
 | |- _ => apply Aux
| |- _=> compute;congruence
end.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros;apply Qmult_lt_0_le_reg_r with q.
elim (Qle_lt_or_eq _ _ H1).
auto.
intro H';symmetry in H';elim (H2 H').
rewrite H.
rewrite Qmult_0_l.
auto.
Qed.


Section Poly.
Variables  x1 x2 x3 x4 :Q.
(* POLYNOMIAL *)
Definition poly :=  x3^8 + x1^4*x2^2*x4^2 - 3*x1^2*x2^2*x3^2*x4^2 + x2^4*x3^2*x4^2 + x1^2*x3^4*x4^2 .

(*SOLUTION *)

(* NUM *)
(* 8522106*x1^2*x3^8 + 8522106*x2^2*x3^8 + 25101076*x3^10 + 8522106*x1^6*x2^2*x4^2 + 8522106*x1^4*x2^4*x4^2 - 465242*x1^4*x2^2*x3^2*x4^2 - 17044212*x1^2*x2^4*x3^2*x4^2 + 8522106*x2^6*x3^2*x4^2 + 8522106*x1^4*x3^4*x4^2 - 66781122*x1^2*x2^2*x3^4*x4^2 + 25101076*x2^4*x3^4*x4^2 + 25101076*x1^2*x3^6*x4^2 *)
(* summands *)
Definition num_decomp :=
( ( 25101076 # 1 ) * ( ( 1 # 1 ) * x3^5 )^2) ::
( ( 8522106 # 1 ) * ( ( 1 # 1 ) * x2*x3^4 )^2) ::
( ( 25101076 # 1 ) * ( ( -1 # 2 ) * x1^2*x2^2*x4 + ( -1 # 2 ) * x1^2*x3^2*x4 + ( 1 # 1 ) * x2^2*x3^2*x4 )^2) ::
( ( 8522106 # 1 ) * ( ( -1 # 1 ) * x1^2*x2*x3*x4 + ( 1 # 1 ) * x2^3*x3*x4 )^2) ::
( ( 25101076 # 1 ) * ( ( -1 # 1 ) * x1*x2^2*x3*x4 + ( 1 # 1 ) * x1*x3^3*x4 )^2) ::
( ( 8522106 # 1 ) * ( ( 1 # 1 ) * x1*x3^4 )^2) ::
( ( 8522106 # 1 ) * ( ( -1 # 1 ) * x1^3*x2*x4 + ( 1 # 1 ) * x1*x2*x3^2*x4 )^2) ::
( ( 2246837 # 1 ) * ( ( -1 # 1 ) * x1^2*x2^2*x4 + ( 1 # 1 ) * x1^2*x3^2*x4 )^2) ::
nil.

(* DEN *)
(* 8522106*x1^2 + 8522106*x2^2 + 25101076*x3^2 *)
(* summands *)
Definition denum_decomp :=
( ( 25101076 # 1 ) * ( ( 1 # 1 ) * x3 )^2) ::
( ( 8522106 # 1 ) * ( ( 1 # 1 ) * x2 )^2) ::
( ( 8522106 # 1 ) * ( ( 1 # 1 ) * x1 )^2) ::
nil.

Lemma Ident: poly * Qsum (denum_decomp) == Qsum (num_decomp).
intros;simpl.
unfold poly.
Time ring.                            
Qed.

Lemma num_pos: Qsum (num_decomp) >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma  denum_pos: Qsum denum_decomp >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma T:~Qsum denum_decomp == 0 -> poly >=0.
intros.
apply (non_critical _ _ _ Ident);
auto using num_pos,denum_pos.
Qed.

End Poly.


Check T.
