import sys
import re

file = sys.stdin

line=file.readline()
if line != "VARIABLES\n":
  sys.exit("cannot find VARIABLES clause") 
line=file.readline()
variables=line.split()

ring=PolynomialRing(QQ,variables)

line=file.readline()
if line != "POLYNOMIAL\n":
  sys.exit("cannot find POLYNOMIAL clause")
line=file.readline()
polynomial=ring(line)

line=file.readline()
if line != "SOLUTION\n":
  sys.exit("cannot find SOLUTION clause")

def read_sos():
  global line
  pattern=re.compile("^(.*)\*\((.*)\)\^2\n$")
  line=file.readline()
  sos=[]
  if line != "summands\n":
    sys.exit("cannot find summands clause")
  while True:
    line=file.readline()
    match=re.search(pattern,line)
    if match==None:
      break
    (coeff,linear_form) = match.groups()
    sos.append((QQ(coeff), ring(linear_form)))
  return sos

def reassemble_sos(sos):
  return sum([coeff * linear_form^2 for (coeff, linear_form) in sos])

line=file.readline()
if line != "NUM\n":
  sys.exit("cannot find NUM clause")
line=file.readline()
num=ring(line)
num_sos=read_sos()
assert (num == reassemble_sos(num_sos))

if line != "DEN\n":
  sys.exit("cannot find DEN clause")
line=file.readline()
den=ring(line)
den_sos=read_sos()
assert (den == reassemble_sos(den_sos))

assert(polynomial == num/den)

print """Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Definition Qsum l := fold_right Qplus 0%Q l.

Theorem Aux: forall a x, a >=0 -> a*x^2 >=0.
simpl.
intros K X.
intro H0.
rewrite Qmult_comm.
setoid_replace 0 with (0*K).
2:ring.
apply Qmult_le_compat_r;auto.
case (Qlt_le_dec 0 X).
intro H.
assert (H':=Qlt_le_weak _ _ H).
setoid_replace 0 with (0 * X).
2:ring.
apply Qmult_le_compat_r;auto.
intro H.
assert (H':-X >= 0).
setoid_replace 0 with (-0).
2:ring.
apply Qopp_le_compat;auto.
setoid_replace 0 with (0 * -X).
2:ring.
setoid_replace (X * X) with (-X * -X).
2:ring.
apply Qmult_le_compat_r;auto.
Qed.

Theorem Aux2: forall x y, 0 <= x -> 0 <= y -> 0 <= x + y.
intros x y Hx Hy.
setoid_replace 0 with (0+0) at 1; [apply Qplus_le_compat|ring].
exact Hx.
exact Hy.
Qed.

Ltac simplify_sos := match goal with 
 |- 0<= ?a + ?b => apply Aux2
 | |- 0 <= 0 => reflexivity
 | |- _ => apply Aux
| |- _=> compute;congruence
end.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros;apply Qmult_lt_0_le_reg_r with q.
elim (Qle_lt_or_eq _ _ H1).
auto.
intro H';symmetry in H';elim (H2 H').
rewrite H.
rewrite Qmult_0_l.
auto.
Qed.


Section Poly."""

print "Variables ", " ".join(variables), ":Q."

def print_rational(q):
  print "(", q.numerator(), "#", q.denominator(), ")",

def print_linear_form(form):
  print_plus=False
  for (coeff, monomial) in zip(form.coefficients(), form.monomials()):
    if print_plus:
      print "+",
    print_plus=True
    print_rational(coeff)
    print "*", monomial,

def print_sos(sos):
  for (coeff, linear_form) in sos:
    print "(",
    print_rational(coeff)
    print "* (",
    print_linear_form(linear_form),
    print ")^2) ::"
  print "nil."

print "(* POLYNOMIAL *)"
print "Definition poly := ", polynomial, "."
print
print "(*SOLUTION *)"
print
print "(* NUM *)"
print "(*", num, "*)"
print "(* summands *)"
print "Definition num_decomp :="
print_sos(num_sos)
print
print "(* DEN *)"
print "(*", den, "*)"
print "(* summands *)"
print "Definition denum_decomp :="
print_sos(den_sos)
print
print """Lemma Ident: poly * Qsum (denum_decomp) == Qsum (num_decomp).
intros;simpl.
unfold poly.
Time ring.                            
Qed.

Lemma num_pos: Qsum (num_decomp) >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma  denum_pos: Qsum denum_decomp >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma T:~Qsum denum_decomp == 0 -> poly >=0.
intros.
apply (non_critical _ _ _ Ident);
auto using num_pos,denum_pos.
Qed.

End Poly.


Check T."""
