
Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Record SOS_part := mkSp { knum: positive; kdenum:positive ; to_be_squared:Q}.

Definition SOS:= list SOS_part.

Definition eval_SOS (s:SOS) := 
  fold_right (fun (Sp:SOS_part) aq => let(kn,kd,tbs) := Sp in 
  ((Zpos kn#kd) * tbs ^2 +aq ))  0%Q s.

Lemma pos_SOS: forall s:SOS, 0<= eval_SOS s.
intro s.
induction s as [ | [kn kd tbs] s Hs];simpl.
apply Qle_refl.
apply Qle_trans with (0+0)%Q.
simpl;apply Qle_refl.
apply Qplus_le_compat.
destruct tbs as [[ | p | n ] d];compute;congruence.
apply Hs.
Qed.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros [[ |p|p] p'] [[ |q|q] q'] [[ |r|r] r'];unfold Qmult,Qle,Qeq,Zle;simpl;
try congruence.
Qed.

Section Poly.
Variables  x1 x2 x3 x4 x5 :Q.
(* POLYNOMIAL *)
Definition poly :=  x1^4 - 2*x1^2*x2^2 + x2^4 + 2*x1^2*x3^2 - 2*x2^2*x3^2 + x3^4 + 2*x1^2*x4^2 + 2*x2^2*x4^2 - 2*x3^2*x4^2 + x4^4 - 2*x1^2*x5^2 + 2*x2^2*x5^2 + 2*x3^2*x5^2 - 2*x4^2*x5^2 + x5^4 .

(*SOLUTION *)

(* NUM *)
(* 23237900/3*x1^6 - 23237900/3*x1^4*x2^2 - 23237900/3*x1^2*x2^4 + 23237900/3*x2^6 + 23237900*x1^4*x3^2 - 46475800/3*x1^2*x2^2*x3^2 - 23237900/3*x2^4*x3^2 + 23237900*x1^2*x3^4 - 23237900/3*x2^2*x3^4 + 23237900/3*x3^6 + 23237900*x1^4*x4^2 + 46475800/3*x1^2*x2^2*x4^2 + 23237900*x2^4*x4^2 + 46475800/3*x1^2*x3^2*x4^2 - 46475800/3*x2^2*x3^2*x4^2 - 23237900/3*x3^4*x4^2 + 23237900*x1^2*x4^4 + 23237900*x2^2*x4^4 - 23237900/3*x3^2*x4^4 + 23237900/3*x4^6 - 23237900/3*x1^4*x5^2 - 46475800/3*x1^2*x2^2*x5^2 + 23237900*x2^4*x5^2 + 46475800/3*x1^2*x3^2*x5^2 + 46475800/3*x2^2*x3^2*x5^2 + 23237900*x3^4*x5^2 - 46475800/3*x1^2*x4^2*x5^2 + 46475800/3*x2^2*x4^2*x5^2 - 46475800/3*x3^2*x4^2*x5^2 - 23237900/3*x4^4*x5^2 - 23237900/3*x1^2*x5^4 + 23237900*x2^2*x5^4 + 23237900*x3^2*x5^4 - 23237900/3*x4^2*x5^4 + 23237900/3*x5^6 *)
(* summands *)
Definition num_decomp :=
(mkSp 23237900 3 ( ( -1 # 1 ) * x1^2*x5 + ( 1 # 1 ) * x2^2*x5 + ( 1 # 1 ) * x3^2*x5 + ( -1 # 1 ) * x4^2*x5 + ( 1 # 1 ) * x5^3 )) ::
(mkSp 23237900 3 ( ( -1 # 1 ) * x1^2*x4 + ( -1 # 1 ) * x2^2*x4 + ( 1 # 1 ) * x3^2*x4 + ( -1 # 1 ) * x4^3 + ( 1 # 1 ) * x4*x5^2 )) ::
(mkSp 23237900 3 ( ( 1 # 1 ) * x1^2*x3 + ( -1 # 1 ) * x2^2*x3 + ( 1 # 1 ) * x3^3 + ( -1 # 1 ) * x3*x4^2 + ( 1 # 1 ) * x3*x5^2 )) ::
(mkSp 23237900 3 ( ( -1 # 1 ) * x1^2*x2 + ( 1 # 1 ) * x2^3 + ( -1 # 1 ) * x2*x3^2 + ( 1 # 1 ) * x2*x4^2 + ( 1 # 1 ) * x2*x5^2 )) ::
(mkSp 92951600 3 ( ( 1 # 1 ) * x2*x4*x5 )) ::
(mkSp 92951600 3 ( ( 1 # 1 ) * x2*x3*x5 )) ::
(mkSp 23237900 3 ( ( -1 # 1 ) * x1^3 + ( 1 # 1 ) * x1*x2^2 + ( -1 # 1 ) * x1*x3^2 + ( -1 # 1 ) * x1*x4^2 + ( 1 # 1 ) * x1*x5^2 )) ::
(mkSp 92951600 3 ( ( 1 # 1 ) * x1*x3*x5 )) ::
(mkSp 92951600 3 ( ( 1 # 1 ) * x1*x3*x4 )) ::
(mkSp 92951600 3 ( ( 1 # 1 ) * x1*x2*x4 )) ::
nil.

(* DEN *)
(* 23237900/3*x1^2 + 23237900/3*x2^2 + 23237900/3*x3^2 + 23237900/3*x4^2 + 23237900/3*x5^2 *)
(* summands *)
Definition denum_decomp :=
(mkSp 23237900 3 ( ( 1 # 1 ) * x5 )) ::
(mkSp 23237900 3 ( ( 1 # 1 ) * x4 )) ::
(mkSp 23237900 3 ( ( 1 # 1 ) * x3 )) ::
(mkSp 23237900 3 ( ( 1 # 1 ) * x2 )) ::
(mkSp 23237900 3 ( ( 1 # 1 ) * x1 )) ::
nil.

Definition num:=eval_SOS num_decomp.
Definition denum:=eval_SOS denum_decomp.

Lemma Ident: poly * denum == num.
unfold poly, denum,denum_decomp, num,num_decomp.
simpl eval_SOS.
Time ring.
Qed.

Lemma T:~denum == 0 -> poly >=0.
intros.
apply non_critical with num denum.
apply Ident.
apply pos_SOS.
apply pos_SOS.
assumption.
Qed.

End Poly.


Check T.
