
Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Record SOS_part := mkSp { knum: positive; kdenum:positive ; to_be_squared:Q}.

Definition SOS:= list SOS_part.

Definition eval_SOS (s:SOS) := 
  fold_right (fun (Sp:SOS_part) aq => let(kn,kd,tbs) := Sp in 
  ((Zpos kn#kd) * tbs ^2 +aq ))  0%Q s.

Lemma pos_SOS: forall s:SOS, 0<= eval_SOS s.
intro s.
induction s as [ | [kn kd tbs] s Hs];simpl.
apply Qle_refl.
apply Qle_trans with (0+0)%Q.
simpl;apply Qle_refl.
apply Qplus_le_compat.
destruct tbs as [[ | p | n ] d];compute;congruence.
apply Hs.
Qed.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros [[ |p|p] p'] [[ |q|q] q'] [[ |r|r] r'];unfold Qmult,Qle,Qeq,Zle;simpl;
try congruence.
Qed.

Section Poly.
Variables  x y z :Q.
(* POLYNOMIAL *)
Definition poly :=  x^4*y^2 - 3*x^2*y^2*z^2 + y^4*z^2 + x^2*z^4 .

(*SOLUTION *)

(* NUM *)
(* 8128145*x^6*y^2 + 8128146*x^4*y^4 - 1934498*x^4*y^2*z^2 - 16256293*x^2*y^4*z^2 + 8128146*y^6*z^2 + 8128145*x^4*z^4 - 59221665*x^2*y^2*z^4 + 22449937*y^4*z^4 + 22449937*x^2*z^6 *)
(* summands *)
Definition num_decomp :=
(mkSp 22449937 1 ( ( -11224969 # 22449937 ) * x^2*y^2 + ( -11224968 # 22449937 ) * x^2*z^2 + ( 1 # 1 ) * y^2*z^2 )) ::
(mkSp 8128146 1 ( ( -1 # 1 ) * x^2*y*z + ( 1 # 1 ) * y^3*z )) ::
(mkSp 22449937 1 ( ( -1 # 1 ) * x*y^2*z + ( 1 # 1 ) * x*z^3 )) ::
(mkSp 8128145 1 ( ( -1 # 1 ) * x^3*y + ( 1 # 1 ) * x*y*z^2 )) ::
(mkSp 56476436575841 22449937 ( ( -1 # 1 ) * x^2*y^2 + ( 1 # 1 ) * x^2*z^2 )) ::
nil.

(* DEN *)
(* 8128145*x^2 + 8128146*y^2 + 22449937*z^2 *)
(* summands *)
Definition denum_decomp :=
(mkSp 22449937 1 ( ( 1 # 1 ) * z )) ::
(mkSp 8128146 1 ( ( 1 # 1 ) * y )) ::
(mkSp 8128145 1 ( ( 1 # 1 ) * x )) ::
nil.

Definition num:=eval_SOS num_decomp.
Definition denum:=eval_SOS denum_decomp.

Lemma Ident: poly * denum == num.
unfold poly, denum,denum_decomp, num,num_decomp.
simpl eval_SOS.
Time ring.
Qed.

Lemma T:~denum == 0 -> poly >=0.
intros.
apply non_critical with num denum.
apply Ident.
apply pos_SOS.
apply pos_SOS.
assumption.
Qed.

End Poly.


Check T.
