
Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Record SOS_part := mkSp { knum: positive; kdenum:positive ; to_be_squared:Q}.

Definition SOS:= list SOS_part.

Definition eval_SOS (s:SOS) := 
  fold_right (fun (Sp:SOS_part) aq => let(kn,kd,tbs) := Sp in 
  ((Zpos kn#kd) * tbs ^2 +aq ))  0%Q s.

Lemma pos_SOS: forall s:SOS, 0<= eval_SOS s.
intro s.
induction s as [ | [kn kd tbs] s Hs];simpl.
apply Qle_refl.
apply Qle_trans with (0+0)%Q.
simpl;apply Qle_refl.
apply Qplus_le_compat.
destruct tbs as [[ | p | n ] d];compute;congruence.
apply Hs.
Qed.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros [[ |p|p] p'] [[ |q|q] q'] [[ |r|r] r'];unfold Qmult,Qle,Qeq,Zle;simpl;
try congruence.
Qed.

Section Poly.
Variables  x1 x2 x3 x4 :Q.
(* POLYNOMIAL *)
Definition poly :=  x3^8 + x1^4*x2^2*x4^2 - 3*x1^2*x2^2*x3^2*x4^2 + x2^4*x3^2*x4^2 + x1^2*x3^4*x4^2 .

(*SOLUTION *)

(* NUM *)
(* 8522106*x1^2*x3^8 + 8522106*x2^2*x3^8 + 25101076*x3^10 + 8522106*x1^6*x2^2*x4^2 + 8522106*x1^4*x2^4*x4^2 - 465242*x1^4*x2^2*x3^2*x4^2 - 17044212*x1^2*x2^4*x3^2*x4^2 + 8522106*x2^6*x3^2*x4^2 + 8522106*x1^4*x3^4*x4^2 - 66781122*x1^2*x2^2*x3^4*x4^2 + 25101076*x2^4*x3^4*x4^2 + 25101076*x1^2*x3^6*x4^2 *)
(* summands *)
Definition num_decomp :=
(mkSp 25101076 1 ( ( 1 # 1 ) * x3^5 )) ::
(mkSp 8522106 1 ( ( 1 # 1 ) * x2*x3^4 )) ::
(mkSp 25101076 1 ( ( -1 # 2 ) * x1^2*x2^2*x4 + ( -1 # 2 ) * x1^2*x3^2*x4 + ( 1 # 1 ) * x2^2*x3^2*x4 )) ::
(mkSp 8522106 1 ( ( -1 # 1 ) * x1^2*x2*x3*x4 + ( 1 # 1 ) * x2^3*x3*x4 )) ::
(mkSp 25101076 1 ( ( -1 # 1 ) * x1*x2^2*x3*x4 + ( 1 # 1 ) * x1*x3^3*x4 )) ::
(mkSp 8522106 1 ( ( 1 # 1 ) * x1*x3^4 )) ::
(mkSp 8522106 1 ( ( -1 # 1 ) * x1^3*x2*x4 + ( 1 # 1 ) * x1*x2*x3^2*x4 )) ::
(mkSp 2246837 1 ( ( -1 # 1 ) * x1^2*x2^2*x4 + ( 1 # 1 ) * x1^2*x3^2*x4 )) ::
nil.

(* DEN *)
(* 8522106*x1^2 + 8522106*x2^2 + 25101076*x3^2 *)
(* summands *)
Definition denum_decomp :=
(mkSp 25101076 1 ( ( 1 # 1 ) * x3 )) ::
(mkSp 8522106 1 ( ( 1 # 1 ) * x2 )) ::
(mkSp 8522106 1 ( ( 1 # 1 ) * x1 )) ::
nil.

Definition num:=eval_SOS num_decomp.
Definition denum:=eval_SOS denum_decomp.

Lemma Ident: poly * denum == num.
unfold poly, denum,denum_decomp, num,num_decomp.
simpl eval_SOS.
Time ring.
Qed.

Lemma T:~denum == 0 -> poly >=0.
intros.
apply non_critical with num denum.
apply Ident.
apply pos_SOS.
apply pos_SOS.
assumption.
Qed.

End Poly.


Check T.
