Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Definition Qsum l := fold_right Qplus 0%Q l.

Theorem Aux: forall a x, a >=0 -> a*x^2 >=0.
simpl.
intros K X.
intro H0.
rewrite Qmult_comm.
setoid_replace 0 with (0*K).
2:field.
apply Qmult_le_compat_r;auto.
case (Qlt_le_dec 0 X).
intro H.
assert (H':=Qlt_le_weak _ _ H).
setoid_replace 0 with (0 * X).
2:field.
apply Qmult_le_compat_r;auto.
intro H.
assert (H':-X >= 0).
setoid_replace 0 with (-0).
2:field.
apply Qopp_le_compat;auto.
setoid_replace 0 with (0 * -X).
2:field.
setoid_replace (X * X) with (-X * -X).
2:field.
apply Qmult_le_compat_r;auto.
Qed.

Ltac simplify_sos := match goal with 
 |- 0<= ?a + ?b => setoid_replace 0 with (0+0) at 1; [apply Qplus_le_compat|field]
 | |- 0 <= 0 => reflexivity
 | |- _ => apply Aux
| |- _=> compute;congruence
end.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros;apply Qmult_lt_0_le_reg_r with q.
elim (Qle_lt_or_eq _ _ H1).
auto.
intro H';symmetry in H';elim (H2 H').
rewrite H.
rewrite Qmult_0_l.
auto.
Qed.


Section Poly.

Variables x y z:Q. 


(* POLYNOMIAL *)
Definition poly := x^6 - x^4*y^2 - x^2*y^4 + y^6 - x^4*z^2 + 3*x^2*y^2*z^2 - y^4*z^2 -
x^2*z^4 - y^2*z^4 + z^6.

(* SOLUTION *)
(* NUM *)
(*
8128145*x^8 + 14321791*x^6*y^2 - 30578081*x^4*y^4 - 14321791*x^2*y^6 +
22449936*y^8 - 6193646*x^4*y^2*z^2 + 51093518*x^2*y^4*z^2 -
14321791*y^6*z^2 - 16256290*x^4*z^4 - 6193646*x^2*y^2*z^4 -
30578081*y^4*z^4 + 14321791*y^2*z^6 + 8128145*z^8
*)
(* summands *)

Definition num_decomp := 
(8128145 *( 3096823/8128145*x^4 + x^2*y^2 - 11224968/8128145*y^4 -
11224968/8128145*x^2*z^2 + 3096823/8128145*y^2*z^2 + z^4 )^2) :: 
(8128145 *( -y^3*z + y*z^3 )^2) ::
(56476428447696/8128145 *( x^4 - y^4 - x^2*z^2 + y^2*z^2 )^2) ::
(22449936 *( -x^3*z + x*z^3 )^2) ::
(8128145 *( -x^3*y + x*y^3 )^2) :: nil.

(* DEN *)

(* 8128145*x^2 + 22449936*y^2 + 8128145*z^2 *)
(*summands*)

Definition denum_decomp := (8128145 *( z )^2) :: (22449936 *( y )^2) :: (8128145 *( x )^2) :: nil.

Lemma Ident: poly * Qsum (denum_decomp) == Qsum (num_decomp).
intros;simpl.
unfold poly.
Time field.                            
Qed.

Lemma num_pos: Qsum (denum_decomp) >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma  denum_pos: Qsum num_decomp >=0.
intros;simpl.
repeat simplify_sos.
Qed.

Lemma T:~Qsum denum_decomp == 0 -> poly >=0.
intros.
apply (non_critical _ _ _ Ident);
auto using num_pos,denum_pos.
Qed.

End Poly.


Check T.







