# This file was *autogenerated* from the file quotient_of_sos_to_coq.sage.
from sage.all_cmdline import *   # import sage library
_sage_const_2 = Integer(2)
import sys
import re

file = sys.stdin

line=file.readline()
if line != "VARIABLES\n":
  sys.exit("cannot find VARIABLES clause") 
line=file.readline()
variables=line.split()

ring=PolynomialRing(QQ,variables)

line=file.readline()
if line != "POLYNOMIAL\n":
  sys.exit("cannot find POLYNOMIAL clause")
line=file.readline()
polynomial=ring(line)

line=file.readline()
if line != "SOLUTION\n":
  sys.exit("cannot find SOLUTION clause")

def read_sos():
  global line
  pattern=re.compile("^(.*)\*\((.*)\)\^2\n$")
  line=file.readline()
  sos=[]
  if line != "summands\n":
    sys.exit("cannot find summands clause")
  while True:
    line=file.readline()
    match=re.search(pattern,line)
    if match==None:
      break
    (coeff,linear_form) = match.groups()
    sos.append((QQ(coeff), ring(linear_form)))
  return sos

def reassemble_sos(sos):
  return sum([coeff * linear_form**_sage_const_2  for (coeff, linear_form) in sos])

line=file.readline()
if line != "NUM\n":
  sys.exit("cannot find NUM clause")
line=file.readline()
num=ring(line)
num_sos=read_sos()
assert (num == reassemble_sos(num_sos))

if line != "DEN\n":
  sys.exit("cannot find DEN clause")
line=file.readline()
den=ring(line)
den_sos=read_sos()
assert (den == reassemble_sos(den_sos))

assert(polynomial == num/den)

print """
Require Import Setoid.
Require Import List.
Require Import QArith.

Open Scope Z_scope.
Open Scope Q_scope.

Coercion inject_Z: Z>->Q.

Record SOS_part := mkSp { knum: positive; kdenum:positive ; to_be_squared:Q}.

Definition SOS:= list SOS_part.

Definition eval_SOS (s:SOS) := 
  fold_right (fun (Sp:SOS_part) aq => let(kn,kd,tbs) := Sp in 
  ((Zpos kn#kd) * tbs ^2 +aq ))  0%Q s.

Lemma pos_SOS: forall s:SOS, 0<= eval_SOS s.
intro s.
induction s as [ | [kn kd tbs] s Hs];simpl.
apply Qle_refl.
apply Qle_trans with (0+0)%Q.
simpl;apply Qle_refl.
apply Qplus_le_compat.
destruct tbs as [[ | p | n ] d];compute;congruence.
apply Hs.
Qed.

Lemma non_critical : forall p q r:Q, r*q == p -> p>=0 -> q>=0 -> ~ q == 0 ->r >=0.
intros [[ |p|p] p'] [[ |q|q] q'] [[ |r|r] r'];unfold Qmult,Qle,Qeq,Zle;simpl;
try congruence.
Qed.

Section Poly."""

print "Variables ", " ".join(variables), ":Q."

def print_rational(q):
  print "(", q.numerator(), "#", q.denominator(), ")",

def print_linear_form(form):
  print_plus=False
  for (coeff, monomial) in zip(form.coefficients(), form.monomials()):
    if print_plus:
      print "+",
    print_plus=True
    print_rational(coeff)
    print "*", monomial,

def print_sos(sos):
  for (coeff, linear_form) in sos:
    print "(mkSp", coeff.numerator(), coeff.denominator(), "(",
    print_linear_form(linear_form)
    print ")) ::"
  print "nil."

print "(* POLYNOMIAL *)"
print "Definition poly := ", polynomial, "."
print
print "(*SOLUTION *)"
print
print "(* NUM *)"
print "(*", num, "*)"
print "(* summands *)"
print "Definition num_decomp :="
print_sos(num_sos)
print
print "(* DEN *)"
print "(*", den, "*)"
print "(* summands *)"
print "Definition denum_decomp :="
print_sos(den_sos)
print
print """Definition num:=eval_SOS num_decomp.
Definition denum:=eval_SOS denum_decomp.

Lemma Ident: poly * denum == num.
unfold poly, denum,denum_decomp, num,num_decomp.
simpl eval_SOS.
Time ring.
Qed.

Lemma T:~denum == 0 -> poly >=0.
intros.
apply non_critical with num denum.
apply Ident.
apply pos_SOS.
apply pos_SOS.
assumption.
Qed.

End Poly.


Check T."""
