import sys
import re

file = sys.stdin # open("choi_lam_f.log", "r")

line=file.readline()
if line != "VARIABLES\n":
  sys.exit("cannot find VARIABLES clause") 
line=file.readline()
variables=line.split()

ring=PolynomialRing(QQ,variables)
zring=PolynomialRing(ZZ,variables)

line=file.readline()
if line != "POLYNOMIAL\n":
  sys.exit("cannot find POLYNOMIAL clause")
line=file.readline()
polynomial=ring(line)

line=file.readline()
if line != "SOLUTION\n":
  sys.exit("cannot find SOLUTION clause")

def read_sos():
  global line
  pattern=re.compile("^(.*)\*\((.*)\)\^2\n$")
  line=file.readline()
  sos=[]
  if line != "summands\n":
    sys.exit("cannot find summands clause")
  while True:
    line=file.readline()
    match=re.search(pattern,line)
    if match==None:
      break
    (coeff,linear_form) = match.groups()
    sos.append((QQ(coeff), ring(linear_form)))
  return sos

def reassemble_sos(sos):
  return sum([coeff * linear_form^2 for (coeff, linear_form) in sos])

line=file.readline()
if line != "NUM\n":
  sys.exit("cannot find NUM clause")
line=file.readline()
num=ring(line)
num_sos=read_sos()
assert (num == reassemble_sos(num_sos))

if line != "DEN\n":
  sys.exit("cannot find DEN clause")
line=file.readline()
den=ring(line)
den_sos=read_sos()
assert (den == reassemble_sos(den_sos))

assert(polynomial == num/den)

def common_factor(l):
  common1=lcm([q.denominator() for q in l])
  simpl1=[ZZ(q * common1) for q in l]
  common2=gcd(simpl1)
  return (common2/common1), [ZZ(z / common2) for z in simpl1]

def common_factor_sos_item((coeff, q_linear_form)):
  (common, dummy) = common_factor(q_linear_form.coefficients())
  z_linear_form = zring(q_linear_form/common)
  coeff2 = coeff*common^2
  assert coeff2 * z_linear_form^2 == coeff * q_linear_form^2
  return (coeff2, z_linear_form)

def common_factor_sos(sos):
  sos2 = map(common_factor_sos_item,sos)
  (common2, zcoeffs) = common_factor([qcoeff for (qcoeff, z_linear_form) in sos2])
  zsos=[(zcoeff,z_linear_form) for (zcoeff, (qcoeff, z_linear_form)) in zip(zcoeffs,sos2)]
  return common2, zsos

(num_coeff,num_sos_z)=common_factor_sos(num_sos)
(den_coeff,den_sos_z)=common_factor_sos(den_sos)
assert num == num_coeff * reassemble_sos(num_sos_z)
assert den == den_coeff * reassemble_sos(den_sos_z)
alpha = num_coeff / den_coeff

print """Require Import List.
Require Import ZArith.

Open Scope Z_scope.

Definition Zsum l := fold_right Zplus 0%Z l.

Theorem Aux1: forall p x, (Zpos p)*x^2 >=0.
intros p [| q| q];unfold Zge;simpl;congruence.
Qed.

Theorem Aux2: forall a b, a>=0 -> b>=0 -> a+b >=0.
intros [ |a|a] [ |b|b];unfold Zge;simpl;congruence.
Qed.

Theorem Aux3: 0 >=0.
unfold Zge;simpl;congruence.
Qed.

Theorem Aux4: forall p a, a>=0 -> Zpos p * a >=0.
intros p [ |a|a];unfold Zge;simpl;congruence.
Qed.

Lemma non_critical : forall p q r:Z, r*q = p -> p>=0 -> q>=0 -> ~ q = 0 ->r >=0.
intros [ |p|p] [ |q|q] [ |r|r];simpl;
compute;congruence.
Qed.

Section Poly."""

print "Variables ", " ".join(variables), ":Z."

def print_sos(sos):
  for (coeff, linear_form) in sos:
    print "(", coeff, "* (", linear_form, ")^2) ::"
  print "nil."

print "(* POLYNOMIAL *)"
print "Definition poly := ", polynomial, "."
print
print "(*SOLUTION *)"
print
print "(* NUM *)"
print "Definition num_alpha := ", alpha.numerator(), "."
print "(* summands *)"
print "Definition num_decomp :="
print_sos(num_sos_z)
print
print "(* DEN *)"
print "Definition denum_alpha := ", alpha.denominator(), "."
print "(* summands *)"
print "Definition denum_decomp :="
print_sos(den_sos_z)
print
print """Definition num:=num_alpha*Zsum num_decomp.
Definition denum:=denum_alpha*Zsum denum_decomp.

Lemma Ident: poly * denum = num.
unfold poly, denum,denum_alpha, denum_decomp, num,num_alpha, num_decomp, Zsum, fold_right.
Time ring.
Qed.

Ltac simplify_sos := match goal with 
 | |- ?a + ?b >= 0 => apply Aux2;[apply Aux1|simplify_sos]
 | |- 0 >= 0 => apply Aux3 
end.

Lemma  num_pos: num >=0.
unfold num,num_alpha;apply Aux4.
unfold num_decomp,Zsum,fold_right.
simplify_sos.
Qed.

Lemma denum_pos: denum >=0.
unfold num,num_alpha;apply Aux4.
unfold denum_decomp,Zsum,fold_right.
simplify_sos.
Qed.

Lemma T:~denum = 0 -> poly >=0.
intros.
apply non_critical with num denum.
apply Ident.
apply num_pos.
apply denum_pos.
assumption.
Qed.

End Poly.


Check T."""
