# `0 <= a /\ 0 <= b /\ 0 <= c /\ 0 <= d
#   ==> 256 * a * b * c * d <= (a + b + c + d)^4`;;

load 'solve_sos_equation.sage'
load 'prepare_polynomials.sage'

indeterminates=['a','b','c']
polynomial_ring=PolynomialRing(QQ, indeterminates)
  
conseq=polynomial_ring('(a + b + c + 1)^4 - 256 * a * b * c')

rhs_polynomial=polynomial_ring(0)

degree=5
positive_polynomials_pairs=[
  (polynomial_ring(1), monomial_basis_symmetric(degree)),
  (polynomial_ring('a'), [polynomial_ring('a')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('b'), polynomial_ring('c')])]),
  (polynomial_ring('b'), [polynomial_ring('b')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('a'), polynomial_ring('c')])]),
  (polynomial_ring('c'), [polynomial_ring('c')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('b'), polynomial_ring('a')])]),
  (polynomial_ring('b*c'), [polynomial_ring('a')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('b'), polynomial_ring('c')])]),
  (polynomial_ring('c*a'), [polynomial_ring('b')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('a'), polynomial_ring('c')])]),
  (polynomial_ring('a*b'), [polynomial_ring('c')^a*p for a in xrange(degree+1) for p in monomial_basis_wrt_symmetric(degree-a, [polynomial_ring('b'), polynomial_ring('a')])]),
  (polynomial_ring('a*b*c'), monomial_basis_symmetric(degree))
 ]

positive_polynomials = [p for p,m in positive_polynomials_pairs]
positive_monomial_lists = [m for p,m in positive_polynomials_pairs]

positive_polynomials.append(-conseq)
positive_monomial_lists.append(monomial_basis_symmetric(4))

equality_polynomials=[]
equality_monomial_list=monomial_basis(2)
equality_monomial_lists=[equality_monomial_list for p in equality_polynomials]

solution = solve_sos_equation_sos(polynomial_ring, positive_polynomials, positive_monomial_lists, equality_polynomials, equality_monomial_lists, rhs_polynomial)

def apply_sos_solution(pblocks, eblocks):
  assert(len(pblocks) == len(positive_monomial_lists))
  assert(len(pblocks) == len(positive_polynomials))
  assert(len(eblocks) == len(equality_monomial_lists))
  assert(len(eblocks) == len(equality_polynomials))

  total = polynomial_ring(0)
  for (sos, poly, monomials) in zip(pblocks, positive_polynomials, positive_monomial_lists):
    total += poly * apply_sos(sos, monomials)
  for (lin, poly, monomials) in zip(eblocks, equality_polynomials, equality_monomial_lists):
    total += poly * (lin * vector(monomials))
  return total

if solution == ():
  print 'NO SOLUTION'
else:
  (pblocks, eblocks) = solution
  assert(apply_sos_solution(pblocks, eblocks)==rhs_polynomial)
  print 'SOLUTION'
  print 'RHS'
  print rhs_polynomial
  for i in xrange(len(positive_polynomials)):
    print 'POSITIVE POLYNOMIAL', i
    print positive_polynomials[i]
    print 'sos polynomial'
    print apply_sos(pblocks[i], positive_monomial_lists[i])
    print 'summands'
    for coeff, sos in pblocks[i]:
      print coeff, "*(", sum([a*b for a, b in zip(sos, positive_monomial_lists[i])]), ")^2"
  for i in xrange(len(equality_polynomials)):
    print 'ZERO POLYNOMIAL', i
    print equality_polynomials[i]
    print 'factor polynomial'
    print eblocks[i] * vector(equality_monomial_lists[i])
