# 0 <= a /\ 0 <= b /\ 0 <= c /\
#    a^2 + b^2 + c^2 + a * b * c = 4
#    ==> a * b + b * c + c * a - a * b * c <= 2;;

load 'solve_sos_equation.sage'
load 'prepare_polynomials.sage'

indeterminates=['a','b','c']
polynomial_ring=PolynomialRing(QQ, indeterminates)
  
conseq=polynomial_ring('2 - (a * b + b * c + c * a - a * b * c)')

rhs_polynomial=polynomial_ring(0)

positive_monomial_list=monomial_basis(3)
positive_polynomials1=[polynomial_ring('a'), polynomial_ring('b'), polynomial_ring('c')]
positive_polynomials=products01(positive_polynomials1)
positive_monomial_lists=[positive_monomial_list for p in positive_polynomials]
positive_polynomials.append(-conseq)
positive_monomial_lists.append(monomial_basis(3))

equality_polynomials=[polynomial_ring('a^2 + b^2 + c^2 + a * b * c - 4')]
equality_monomial_list=monomial_basis(4)
equality_monomial_lists=[equality_monomial_list for p in equality_polynomials]

solution = solve_sos_equation_sos(polynomial_ring, positive_polynomials, positive_monomial_lists, equality_polynomials, equality_monomial_lists, rhs_polynomial)

def apply_sos_solution(pblocks, eblocks):
  assert(len(pblocks) == len(positive_monomial_lists))
  assert(len(pblocks) == len(positive_polynomials))
  assert(len(eblocks) == len(equality_monomial_lists))
  assert(len(eblocks) == len(equality_polynomials))

  total = polynomial_ring(0)
  for (sos, poly, monomials) in zip(pblocks, positive_polynomials, positive_monomial_lists):
    total += poly * apply_sos(sos, monomials)
  for (lin, poly, monomials) in zip(eblocks, equality_polynomials, equality_monomial_lists):
    total += poly * (lin * vector(monomials))
  return total

if solution == ():
  print 'NO SOLUTION'
else:
  (pblocks, eblocks) = solution
  assert(apply_sos_solution(pblocks, eblocks)==rhs_polynomial)
  print 'SOLUTION'
  print 'RHS'
  print rhs_polynomial
  for i in xrange(len(positive_polynomials)):
    print 'POSITIVE POLYNOMIAL', i
    print positive_polynomials[i]
    print 'sos polynomial'
    print apply_sos(pblocks[i], positive_monomial_lists[i])
    print 'summands'
    for coeff, sos in pblocks[i]:
      print coeff, "*(", sum([a*b for a, b in zip(sos, positive_monomial_lists[i])]), ")^2"
  for i in xrange(len(equality_polynomials)):
    print 'ZERO POLYNOMIAL', i
    print equality_polynomials[i]
    print 'factor polynomial'
    print eblocks[i] * vector(equality_monomial_lists[i])
