load 'solve_sos_equation.sage'
load 'prepare_polynomials.sage'

# Examples from Reznick, Some concrete aspects of Hilbert's 17th problem
if 1==0:
  indeterminates=['x','y']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('(11*x+y)^4')
  exponent = 0
  kernel_scaling = 1e14

# Motzkin M
if 1==1:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^4 * y^2 + x^2 * y^4 + z^6 - 3 * x^2 * y^2 * z^2')
  kernel_scaling = 1e14
  exponent = 2

# Robinson R
if 1==0:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^6 + y^6 + z^6 - (x^4 * y^2 + x^2 * y^4 + x^4 * z^2 + x^2 * z^4 + y^4 * z^2 + y^2 * z^4) + 3 * x^2 * y^2 * z^2')
  den_degree = 1

# Robinson f
if 1==0:
  indeterminates=['x','y','z','w']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^2 * (x-w)^2 + y^2 * (y-w)^2 + z^2 * (z-w)^2 + 2*x*y*z * (x+y+z-2*w)')
  den_degree = 1

# Choi-Lam F works (with scaling factor 10^15 or nonzero init)
if 1==0:
  indeterminates=['x1','x2','x3','y1','y2','y3']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x1^2 * y1^2 + x2^2 * y2^2 + x3^2 * y3^2 + 2*(x1^2 * y2^2 + x2^2 * y3^2 + x3^2 * y1^2) - 2*x1*x2*y1*y2 - 2*x1*x3*y1*y3 - 2*x2*x3*y2*y3')
  den_degree=1

# Choi-Lam Q
if 1==0:
  indeterminates=['x', 'y', 'z', 'w']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^2*y^2 + x^2*z^2 + y^2*z^2 + w^4 -4*w*x*y*z')
  den_degree=1

# Choi-Lam S
if 1==0:
  indeterminates=['x', 'y', 'z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x^4 * y^2 + y^4 * z^2 + z^4 * x^2 - 3*x^2*y^2*z^2')
  den_degree=1

# Choi-Lam H
if 1==0:
  indeterminates=['x1', 'x2', 'x3', 'x4', 'x5']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('(x1^2 + x2^2 + x3^2 + x4^2 + x5^2)^2 - 4* (x1^2 * x2^2 + x2^2 * x3^2 + x3^2 * x4^2 + x4^2 * x5^2 + x5^2 * x1^2)');
  den_degree = 1
    
# Schmuedgen q
if 1==0:
  indeterminates=['x','y','z']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('200*(x^3-4*x*z^2)^2 + 200*(y^3-4*y*z^2)^2 + (y^2-x^2)*x*(x+2*z)*(x^2-2*x*z+2*y^2-8*z^2)')
  den_degree=1

# Delzell
if 1==0:
  indeterminates=['x1','x2','x3','x4']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x1^4 * x2^2 * x4^2 + x2^4 * x3^2 * x4^2 + x1^2 * x3^4 * x4^2 -3 * x1^2 * x2^2 * x3^2 * x4^2 + x3^8')
  den_degree=1

# Lax-Lax
if 1==0:
  indeterminates=['x1','x2','x3','x4']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('x1*x2*x3*x4') - sum([polynomial_ring(xi) * prod([polynomial_ring(xj)-polynomial_ring(xi) for xj in indeterminates if xj!=xi]) for xi in indeterminates])
  den_degree=1

# Leep-Starr2
if 1==0:
  indeterminates=['x1','x2']
  polynomial_ring=PolynomialRing(QQ, indeterminates)
  polynomial = polynomial_ring('8 + 1/2 * x1^2 * x2^4 + x1^2 * x2^3 - 2 * x1^3 * x2^3 + 2 * x1 * x2^2 + 10 * x1^2 * x2^2 + 4 * x1^3 * x2^2 + 3 * x1^4 * x2^2 + 4 * x1 * x2 -8 * x1^2 * x2')
  den_degree=1

# assert(polynomial.is_homogeneous())
assert(polynomial.degree() % 2 == 0)

rhs_polynomial = polynomial^(2*exponent+1)
degree=rhs_polynomial.degree()/2
polytope = (rhs_polynomial.newton_polytope())*(1/2)

if rhs_polynomial.is_homogeneous():
  monomials1=monomial_basis_homogeneous(degree)
else:
  monomials1=monomial_basis(degree)

monomials=[m for m in monomials1 if polytope.contains(m.exponents()[0])]
print "MONOMIALS", monomials

solution= solve_sos_equation_sos(polynomial_ring, [1], [monomials], [], [], rhs_polynomial)

print solution
if solution == ():
  print 'NO SOLUTION'
else:
  (solution_q,solution_e) = solution
  num = apply_sos(solution_q[0], monomials)
  print 'SOLUTION'
  print 'NUM'
  print num
  print 'summands'
  for coeff, sos in solution_q[0]:
    print coeff, "*(", sum([a*b for a, b in zip(sos, monomials)]), ")^2"
